/* Adds a value to 30 bytes starting at a specified offset.
 * Public domain code by Norbert, 2016-11-08.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define EXIT_NORMAL 0
#define EXIT_ERROR 1
#define PROGRAM_NAME "hexmover"
#define MAX_PATHFILE 100
#define NR_BYTES_TO_CHANGE 30

#ifndef O_BINARY
#define O_BINARY 0
#endif

/*****************************************************************************/
int main (int argc, char *argv[])
/*****************************************************************************/
{
	char sIn[MAX_PATHFILE + 2];
	char sOut[MAX_PATHFILE + 2];
	int iOffset;
	int iAdd;
	int iFd;
	int iRead;
	char sRead[1 + 2];
	int iEOF;
	int iWrote;

	/*** For copying. ***/
	FILE *fIn;
	FILE *fOut;
	int iData;

	if (argc != 5)
	{
		printf ("Usage: ./%s <in> <out> <offset> <add>\n"
			"\t<in> = Existing in-file.\n"
			"\t<out> = Non-existing out-file.\n"
			"\t<offset> = Where to add.\n"
			"\t<add> = Amount to add.\n\n",
			PROGRAM_NAME);
		exit (EXIT_NORMAL);
	} else {
		snprintf (sIn, MAX_PATHFILE, "%s", argv[1]);
		snprintf (sOut, MAX_PATHFILE, "%s", argv[2]);
		if (access (sOut, R_OK) != -1)
		{
			printf ("[FAILED] File \"%s\" already exists!\n", sOut);
			exit (EXIT_ERROR);
		}
		iOffset = atoi (argv[3]);
		iAdd = atoi (argv[4]);
	}

	/*** First, copy the entire file. ***/
	fIn = fopen (sIn, "rb");
	if (fIn == NULL)
	{
		printf ("[FAILED] Could not open \"%s\": %s!\n",
			sIn, strerror (errno));
		exit (EXIT_ERROR);
	}
	fOut = fopen (sOut, "wb");
	if (fOut == NULL)
	{
		printf ("[FAILED] Could not open \"%s\": %s!\n",
			sOut, strerror (errno));
		exit (EXIT_ERROR);
	}
	while (1)
	{
		iData = fgetc (fIn);
		if (iData == EOF) { break; }
			else { putc (iData, fOut); }
	}
	fclose (fIn);
	fclose (fOut);

	iFd = open (sOut, O_RDWR|O_BINARY);
	if (iFd == -1)
	{
		printf ("[FAILED] Could not open \"%s\": %s!\n",
			sOut, strerror (errno));
		exit (EXIT_ERROR);
	}
	iEOF = 0;
	iWrote = 0;
	do {
		lseek (iFd, iOffset + iWrote, SEEK_SET);
		iRead = read (iFd, sRead, 1);
		lseek (iFd, iOffset + iWrote, SEEK_SET);
		switch (iRead)
		{
			case -1:
				printf ("[FAILED] Could not read from \"%s\": %s!\n",
					sIn, strerror (errno));
				exit (EXIT_ERROR);
				break;
			case 0: iEOF = 1; break;
			default:
				sRead[0] = sRead[0] + iAdd;
				write (iFd, sRead, 1);
				iWrote++;
				break;
		}
	} while ((iEOF == 0) && (iWrote < NR_BYTES_TO_CHANGE));

	close (iFd);

	exit (EXIT_NORMAL);
}
/*****************************************************************************/
