/* Finds the offset of the levels table that is used when loading levels.
 * Public domain code by Norbert, 2016-11-08.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

#define EXIT_NORMAL 0
#define EXIT_ERROR 1
#define PROGRAM_NAME "hexoffset"
#define MAX_PATHFILE 100

#ifndef O_BINARY
#define O_BINARY 0
#endif

/*****************************************************************************/
int main (int argc, char *argv[])
/*****************************************************************************/
{
	char sIn[MAX_PATHFILE + 2];
	int iFd;
	int iRead;
	unsigned char sRead[8 + 2];
	int iEOF;
	int iOffset;
	int iOne, iTwo, iThree, iFour;
	char sOne[10], sTwo[10], sThree[10], sFour[10];
	int iMinusLoop;

	if (argc != 2)
	{
		printf ("Usage: ./%s <in>\n"
			"\t<in> = Existing in-file.\n\n",
			PROGRAM_NAME);
		exit (EXIT_NORMAL);
	} else {
		snprintf (sIn, MAX_PATHFILE, "%s", argv[1]);
	}

	iFd = open (sIn, O_RDONLY|O_BINARY);
	if (iFd == -1)
	{
		printf ("[FAILED] Could not open \"%s\": %s!\n",
			sIn, strerror (errno));
		exit (EXIT_ERROR);
	}

	iEOF = 0;

	iOffset = 0;
	do {
		lseek (iFd, iOffset, SEEK_SET);
		iRead = read (iFd, sRead, 8);
		switch (iRead)
		{
			case -1:
				printf ("[FAILED] Could not read from \"%s\": %s!\n",
					sIn, strerror (errno));
				exit (EXIT_ERROR);
				break;
			case 0: iEOF = 1; break;
			default:
				snprintf (sOne, 10, "%02x%02x", sRead[1], sRead[0]);
				iOne = strtoul (sOne, NULL, 16);
				snprintf (sTwo, 10, "%02x%02x", sRead[3], sRead[2]);
				iTwo = strtoul (sTwo, NULL, 16);
				snprintf (sThree, 10, "%02x%02x", sRead[5], sRead[4]);
				iThree = strtoul (sThree, NULL, 16);
				snprintf (sFour, 10, "%02x%02x", sRead[7], sRead[6]);
				iFour = strtoul (sFour, NULL, 16);
				for (iMinusLoop = 0; iMinusLoop < 0xFFFF; iMinusLoop++)
				{
					/* These values are the first four compressed level sizes, to
					 * measure cumulative distances from a starting offset. You can
					 * obtain these sizes by running:
					 * $ ./legbop -d|grep "Compressed level size"
					 */
					if ((iOne - iMinusLoop == 108) &&
						(iTwo - iMinusLoop == 108 + 585) &&
						(iThree - iMinusLoop == 108 + 585 + 584) &&
						(iFour - iMinusLoop == 108 + 585 + 584 + 621))
					{
						printf ("[ INFO ] Found at offset 0x%02X, all values"
							" increased by 0x%02X.\n", iOffset - 2, iMinusLoop);
						exit (EXIT_NORMAL);
					}
				}
				break;
		}
		iOffset++;
	} while (iEOF == 0);

	close (iFd);

	exit (EXIT_NORMAL);
}
/*****************************************************************************/
