/* Searches for specific 30-tile room layouts, in NON-compressed levels.
 * Public domain code by Norbert, 2016-11-08.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#define EXIT_NORMAL 0
#define EXIT_ERROR 1
#define PROGRAM_NAME "hexsearch"
#define MAX_PATHFILE 100
#define NR_TILES 30
#define MAX_BYTES 4

#ifndef O_BINARY
#define O_BINARY 0
#endif

/*****************************************************************************/
int main (int argc, char *argv[])
/*****************************************************************************/
{
	char sIn[MAX_PATHFILE + 2];
	char sChars[100];
	FILE *fIn;
	char sRead[(NR_TILES * 4) + 2];
	int iOffset;
	int iBytes, iTile, iTileO;
	char cTile[NR_TILES + 2][MAX_BYTES + 2];
	int iBingo;
	int iByte;
	int iBasic;
	char cOne, cTwo;

	if (argc != 3)
	{
		printf ("Usage: ./%s <file> <chars>\n"
			"\t<file> = Existing in-file.\n"
			"\t<chars> = Thirty characters.\n\n"
			"Example for DOS (level 1, room 1):\n"
			"sssfzfzfwwttzpswwwwwwwwwrplzfw\n"
			"Possible examples for GBC (both level 1, room 1):\n"
			"sssfzfzfWwttfpsWwwwwwwwwrplzfW\n"
			"12234545678990a677777777b0cd56\n\n",
			PROGRAM_NAME);
		exit (EXIT_NORMAL);
	} else if (strlen (argv[2]) != 30) {
		printf ("[FAILED] These are not 30 characters!\n");
		exit (EXIT_ERROR);
	} else {
		snprintf (sIn, MAX_PATHFILE, "%s", argv[1]);
		snprintf (sChars, 100, "%s", argv[2]);
	}

	fIn = fopen (sIn, "rb");
	if (fIn == NULL)
	{
		printf ("[FAILED] Could not open \"%s\": %s!\n",
			sIn, strerror (errno));
		exit (EXIT_ERROR);
	}

	iOffset = 0;
	while (1)
	{
		fseek (fIn, iOffset, SEEK_SET);
		if (fgets (sRead, NR_TILES * 4, fIn) == NULL)
		{
			break;
		} else {
			for (iBytes = 1; iBytes <= MAX_BYTES; iBytes++)
			{
				for (iTile = 1; iTile <= NR_TILES; iTile++)
				{
					cTile[iTile][0] = sRead[((iTile - 1) * iBytes) + 0];
					if (iBytes > 1)
						{ cTile[iTile][1] = sRead[((iTile - 1) * iBytes) + 1]; }
					if (iBytes > 2)
						{ cTile[iTile][2] = sRead[((iTile - 1) * iBytes) + 2]; }
					if (iBytes > 3)
						{ cTile[iTile][3] = sRead[((iTile - 1) * iBytes) + 3]; }

					/*** We never want values over 0x1F. ***/
					while (cTile[iTile][0] >= 32) { cTile[iTile][0]-=32; }
					while (cTile[iTile][1] >= 32) { cTile[iTile][1]-=32; }
					while (cTile[iTile][2] >= 32) { cTile[iTile][2]-=32; }
					while (cTile[iTile][3] >= 32) { cTile[iTile][3]-=32; }
				}

				iBingo = 1;

				for (iTile = 1; iTile <= NR_TILES; iTile++)
				{
					for (iTileO = 1; iTileO <= NR_TILES; iTileO++)
					{
						for (iByte = 1; iByte <= iBytes; iByte++)
						{
							if ((cTile[iTile][iByte - 1] != cTile[iTileO][iByte - 1]) &&
								(sChars[iTile - 1] == sChars[iTileO - 1]))
							{ iBingo = 0; }
						}
					}
				}

				/* Do not include hits where all tiles are
				 * either 0x00, 0x01, 0xff or -0x01.
				 */
				iBasic = 1;
				for (iByte = 1; iByte <= iBytes; iByte++)
				{
					for (iTile = 1; iTile <= NR_TILES; iTile++)
					{
						if ((cTile[iTile][iByte - 1] != 0x00) &&
							(cTile[iTile][iByte - 1] != 0x01) &&
							(cTile[iTile][iByte - 1] != 0xff) &&
							(cTile[iTile][iByte - 1] != -0x01)) { iBasic = 0; }
					}
				}
				if (iBasic == 1) { iBingo = 0; }

				/* Do not include hits with only one or two tile types. ***/
				cOne = '\0';
				cTwo = '\0';
				iBasic = 1;
				for (iByte = 1; iByte <= iBytes; iByte++)
				{
					for (iTile = 1; iTile <= NR_TILES; iTile++)
					{
						if (cOne == '\0')
						{
							cOne = cTile[iTile][iByte - 1];
						} else if (cOne == cTile[iTile][iByte - 1]) {
							/*** We already know about this tile type. ***/
						} else if (cTwo == '\0') {
							cTwo = cTile[iTile][iByte - 1];
						} else if (cTwo == cTile[iTile][iByte - 1]) {
							/*** We already know about this tile type. ***/
						} else {
							iBasic = 0;
						}
					}
				}
				if (iBasic == 1) { iBingo = 0; }

				/*** Do not include hits where more than four tiles are 0x00. ***/
				iBasic = 0;
				for (iByte = 1; iByte <= iBytes; iByte++)
				{
					for (iTile = 1; iTile <= NR_TILES; iTile++)
					{
						if (cTile[iTile][iByte - 1] == 0x00) { iBasic++; }
					}
				}
				if ((iBasic / iBytes) > 4) { iBingo = 0; }

				if (iBingo == 1)
				{
					printf ("[ INFO ] Found at offset 0x%02X (%i), each tile is %i"
						" byte(s):\n", iOffset, iOffset, iBytes);
					for (iByte = 1; iByte <= iBytes; iByte++)
					{
						for (iTile = 1; iTile <= NR_TILES; iTile++)
						{
							printf ("%02X ", cTile[iTile][iByte - 1]);
						}
					}
					printf ("\n");
				}
			}
		}
		iOffset++;
	}

	fclose (fIn);

	exit (EXIT_NORMAL);
}
/*****************************************************************************/
