/* Possible offsets for two neighboring torches, in NON-compressed levels.
 * Public domain code by Norbert, 2016-11-08.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

#define EXIT_NORMAL 0
#define EXIT_ERROR 1
#define PROGRAM_NAME "hextorches"
#define MAX_PATHFILE 100

#ifndef O_BINARY
#define O_BINARY 0
#endif

int IsTorch (char sByte);

/*****************************************************************************/
int main (int argc, char *argv[])
/*****************************************************************************/
{
	char sIn[MAX_PATHFILE + 2];
	int iFd;
	int iRead;
	unsigned char sRead[2 + 2];
	int iEOF;
	int iOffset;
	int iCount;

	if (argc != 2)
	{
		printf ("Usage: ./%s <in>\n"
			"\t<in> = Existing in-file.\n\n",
			PROGRAM_NAME);
		exit (EXIT_NORMAL);
	} else {
		snprintf (sIn, MAX_PATHFILE, "%s", argv[1]);
	}

	iFd = open (sIn, O_RDONLY|O_BINARY);
	if (iFd == -1)
	{
		printf ("[FAILED] Could not open \"%s\": %s!\n",
			sIn, strerror (errno));
		exit (EXIT_ERROR);
	}

	iCount = 0;
	iEOF = 0;

	iOffset = 0;
	do {
		lseek (iFd, iOffset, SEEK_SET);
		iRead = read (iFd, sRead, 2);
		switch (iRead)
		{
			case -1:
				printf ("[FAILED] Could not read from \"%s\": %s!\n",
					sIn, strerror (errno));
				exit (EXIT_ERROR);
				break;
			case 0: iEOF = 1; break;
			default:
				if ((IsTorch (sRead[0]) == 1) && (IsTorch (sRead[1]) == 1))
				{
					printf ("[ INFO ] Possible hit: 0x%02X (%i)\n", iOffset, iOffset);
					iCount++;
				}
				break;
		}
		iOffset++;
	} while (iEOF == 0);

	printf ("[ INFO ] Total: %i hits.\n", iCount);

	close (iFd);

	exit (EXIT_NORMAL);
}
/*****************************************************************************/
int IsTorch (char sByte)
/*****************************************************************************/
{
	if ((sByte == 0x13) || (sByte == 0x26) ||
		(sByte == 0x27) || (sByte == 0x33) ||
		(sByte == 0x4C) || (sByte == 0x4D) ||
		(sByte == 0x4E) || (sByte == 0x4F) ||
		(sByte == 0x53) || (sByte == 0x73) ||
		(sByte == 0x93) || (sByte == 0x98) ||
		(sByte == 0x99) || (sByte == 0x9A) ||
		(sByte == 0x9B) || (sByte == 0x9C) ||
		(sByte == 0x9D) || (sByte == 0x9E) ||
		(sByte == 0x9F) || (sByte == 0xB3) ||
		(sByte == 0xD3) || (sByte == 0xF3))
	{ return (1); }

	return (0);
}
/*****************************************************************************/
